@@ -10,24 +10,18 @@ https://gitlab.com/keatontaylor/alexapy
 import asyncio
 import json
 import logging
-from typing import (
-    Any,
-    Dict,
-    List,  # noqa pylint: disable=unused-import
-    Optional,
-    Text,
-)
-
-from aiohttp import ClientResponse
-from yarl import URL
+from typing import Any, Dict, Optional, Text
+from typing import List  # noqa pylint: disable=unused-import
 
+from alexapy.aiohttp import ClientConnectionError, ClientResponse
 import backoff
+from yarl import URL
 
 from .alexalogin import AlexaLogin
 from .errors import (
+    AlexapyConnectionError,
     AlexapyLoginError,
     AlexapyTooManyRequestsError,
-    AlexapyConnectionError,
 )
 from .helpers import _catch_all_exceptions, hide_email
 
@@ -45,8 +39,8 @@ class AlexaAPI:
     """
 
     devices: Dict[Text, Any] = {}
-    _sequence_queue: List[Dict[Any, Any]] = []
-    _sequence_lock = asyncio.Lock()
+    _sequence_queue: Dict[Any, List[Dict[Any, Any]]] = {}
+    _sequence_lock: Dict[Any, asyncio.Lock] = {}
 
     def __init__(self, device, login: AlexaLogin):
         """Initialize Alexa device."""
@@ -55,6 +49,8 @@ class AlexaAPI:
         self._session = login.session
         self._url: Text = "https://alexa." + login.url
         self._login._headers["Referer"] = "{}/spa/index.html".format(self._url)
+        AlexaAPI._sequence_queue[self._login.email] = []
+        AlexaAPI._sequence_lock[self._login.email] = asyncio.Lock()
         try:
             assert self._login._cookies is not None
             csrf = self._login._cookies["csrf"]
@@ -70,12 +66,11 @@ class AlexaAPI:
 
     @backoff.on_exception(
         backoff.expo,
-        (AlexapyTooManyRequestsError, AlexapyConnectionError),
+        (AlexapyTooManyRequestsError, AlexapyConnectionError, ClientConnectionError),
         max_time=60,
         max_tries=5,
         logger=__name__,
     )
-    @_catch_all_exceptions
     async def _request(
         self,
         method: Text,
@@ -111,25 +106,21 @@ class AlexaAPI:
             raise AlexapyTooManyRequestsError(response.reason)
         return response
 
-    @_catch_all_exceptions
     async def _post_request(
         self, uri: Text, data: Optional[Dict[Text, Text]] = None
     ) -> ClientResponse:
         return await self._request("post", uri, data)
 
-    @_catch_all_exceptions
     async def _put_request(
         self, uri: Text, data: Optional[Dict[Text, Text]] = None
     ) -> ClientResponse:
         return await self._request("put", uri, data)
 
-    @_catch_all_exceptions
     async def _get_request(
         self, uri: Text, data: Optional[Dict[Text, Text]] = None
     ) -> ClientResponse:
         return await self._request("get", uri, data)
 
-    @_catch_all_exceptions
     async def _del_request(
         self, uri: Text, data: Optional[Dict[Text, Text]] = None
     ) -> ClientResponse:
@@ -138,12 +129,11 @@ class AlexaAPI:
     @staticmethod
     @backoff.on_exception(
         backoff.expo,
-        (AlexapyTooManyRequestsError, AlexapyConnectionError),
+        (AlexapyTooManyRequestsError, AlexapyConnectionError, ClientConnectionError),
         max_time=60,
         max_tries=5,
         logger=__name__,
     )
-    @_catch_all_exceptions
     async def _static_request(
         method: Text,
         login: AlexaLogin,
@@ -180,6 +170,7 @@ class AlexaAPI:
             raise AlexapyTooManyRequestsError(response.reason)
         return response
 
+    @_catch_all_exceptions
     async def run_behavior(self, node_data, queue_delay: float = 1.5,) -> None:
         """Queue node_data for running a behavior in sequence.
 
@@ -202,42 +193,50 @@ class AlexaAPI:
                 "@type": "com.amazon.alexa.behaviors.model.SerialNode",
                 "nodesToExecute": [],
             }
-            if AlexaAPI._sequence_queue:
-                last_node = AlexaAPI._sequence_queue[-1]
-                new_node = node_data
-                if node_data and isinstance(node_data, list):
-                    new_node = node_data[0]
-                if (
-                    last_node.get("operationPayload", {}).get("deviceSerialNumber")
-                    and new_node.get("operationPayload", {}).get("deviceSerialNumber")
-                ) and last_node.get("operationPayload", {}).get(
-                    "deviceSerialNumber"
-                ) != new_node.get(
-                    "operationPayload", {}
-                ).get(
-                    "deviceSerialNumber"
-                ):
-                    _LOGGER.debug("Creating Parallel node")
-                    sequence_json["startNode"][
-                        "@type"
-                    ] = "com.amazon.alexa.behaviors.model.ParallelNode"
-            if isinstance(node_data, list):
-                AlexaAPI._sequence_queue.extend(node_data)
-            else:
-                AlexaAPI._sequence_queue.append(node_data)
-            items = len(AlexaAPI._sequence_queue)
+            async with AlexaAPI._sequence_lock[self._login.email]:
+                if AlexaAPI._sequence_queue[self._login.email]:
+                    last_node = AlexaAPI._sequence_queue[self._login.email][-1]
+                    new_node = node_data
+                    if node_data and isinstance(node_data, list):
+                        new_node = node_data[0]
+                    if (
+                        last_node.get("operationPayload", {}).get("deviceSerialNumber")
+                        and new_node.get("operationPayload", {}).get(
+                            "deviceSerialNumber"
+                        )
+                    ) and last_node.get("operationPayload", {}).get(
+                        "deviceSerialNumber"
+                    ) != new_node.get(
+                        "operationPayload", {}
+                    ).get(
+                        "deviceSerialNumber"
+                    ):
+                        _LOGGER.debug("Creating Parallel node")
+                        sequence_json["startNode"][
+                            "@type"
+                        ] = "com.amazon.alexa.behaviors.model.ParallelNode"
+                if isinstance(node_data, list):
+                    AlexaAPI._sequence_queue[self._login.email].extend(node_data)
+                else:
+                    AlexaAPI._sequence_queue[self._login.email].append(node_data)
+                items = len(AlexaAPI._sequence_queue[self._login.email])
+                old_sequence: List[Dict[Any, Any]] = AlexaAPI._sequence_queue[
+                    self._login.email
+                ]
             await asyncio.sleep(queue_delay)
-            if items == len(AlexaAPI._sequence_queue):
-                sequence_json["startNode"]["nodesToExecute"].extend(
-                    AlexaAPI._sequence_queue
-                )
-                AlexaAPI._sequence_queue = []
-                _LOGGER.debug("Creating sequence for %s items", items)
-            else:
-                _LOGGER.debug(
-                    "Queue size changed while waiting %s seconds", queue_delay
-                )
-                return
+            async with AlexaAPI._sequence_lock[self._login.email]:
+                if (
+                    items == len(AlexaAPI._sequence_queue[self._login.email])
+                    and old_sequence == AlexaAPI._sequence_queue[self._login.email]
+                ):
+                    sequence_json["startNode"]["nodesToExecute"].extend(
+                        AlexaAPI._sequence_queue[self._login.email]
+                    )
+                    AlexaAPI._sequence_queue[self._login.email] = []
+                    _LOGGER.debug("Creating sequence for %s items", items)
+                else:
+                    _LOGGER.debug("Queue changed while waiting %s seconds", queue_delay)
+                    return
         data = {
             "behaviorId": "PREVIEW",
             "sequenceJson": json.dumps(sequence_json),
@@ -246,6 +245,7 @@ class AlexaAPI:
         _LOGGER.debug("Running behavior with data: %s", json.dumps(data))
         await self._post_request("/api/behaviors/preview", data=data)
 
+    @_catch_all_exceptions
     async def send_sequence(
         self, sequence: Text, queue_delay: float = 1.5, **kwargs
     ) -> None:
@@ -287,17 +287,16 @@ class AlexaAPI:
 
         """
         operation_payload = {
-            "devices": [
-                {
-                    "deviceType": self._device._device_type,
-                    "deviceSerialNumber": self._device.unique_id,
-                }
-            ],
+            "deviceType": self._device._device_type,
+            "deviceSerialNumber": self._device.unique_id,
             "locale": (self._device._locale if self._device._locale else "en-US"),
             "customerId": self._device._device_owner_customer_id,
         }
         if kwargs is not None:
             operation_payload.update(kwargs)
+            if kwargs.get("devices"):
+                operation_payload.pop("deviceType")
+                operation_payload.pop("deviceSerialNumber")
         node_data = {
             "@type": "com.amazon.alexa.behaviors.model.OpaquePayloadOperationNode",
             "type": sequence,
@@ -305,6 +304,7 @@ class AlexaAPI:
         }
         await self.run_behavior(node_data, queue_delay=queue_delay)
 
+    @_catch_all_exceptions
     async def run_skill(self, skill_id: Text, queue_delay: float = 0) -> None:
         """Run Alexa skill.
 
@@ -337,6 +337,7 @@ class AlexaAPI:
         }
         await self.run_behavior(node_data, queue_delay=queue_delay)
 
+    @_catch_all_exceptions
     async def run_routine(self, utterance: Text, queue_delay: float = 1.5) -> None:
         """Run Alexa automation routine.
 
@@ -412,6 +413,7 @@ class AlexaAPI:
             _populate_device_info(sequence["startNode"])
             await self.run_behavior(sequence["startNode"], queue_delay=queue_delay)
 
+    @_catch_all_exceptions
     async def play_music(
         self,
         provider_id: Text,
@@ -429,6 +431,7 @@ class AlexaAPI:
             queue_delay=queue_delay,
         )
 
+    @_catch_all_exceptions
     async def play_sound(
         self, sound_string_id: Text, customer_id: Text = None, queue_delay: float = 1.5
     ) -> None:
@@ -441,6 +444,7 @@ class AlexaAPI:
             queue_delay=queue_delay,
         )
 
+    @_catch_all_exceptions
     async def stop(
         self,
         customer_id: Text = None,
@@ -467,6 +471,13 @@ class AlexaAPI:
                     "deviceSerialNumber": "ALEXA_ALL_DSN",
                 },
             )
+        else:
+            kwargs["devices"] = [
+                {
+                    "deviceSerialNumber": self._device.unique_id,
+                    "deviceType": self._device._device_type,
+                },
+            ]
 
         await self.send_sequence(
             "Alexa.DeviceControls.Stop",
@@ -519,6 +530,7 @@ class AlexaAPI:
             )
         return devices
 
+    @_catch_all_exceptions
     async def send_tts(
         self,
         message: Text,
@@ -533,7 +545,9 @@ class AlexaAPI:
         should be used instead.
 
         Args:
-        message (string): The message to speak
+        message (string): The message to speak. For canned messages, the message
+                            must start with `alexa.cannedtts.speak` as discovered
+                            in the routines.
         customer_id (string): CustomerId to use for authorization. When none
                              specified this defaults to the device owner. Used
                              with households where others may have their own
@@ -551,16 +565,29 @@ class AlexaAPI:
                                           Must be positive.
 
         """
-        target = {"customerId": customer_id, "devices": self.process_targets(targets)}
-        await self.send_sequence(
-            "Alexa.Speak",
-            customerId=customer_id,
-            textToSpeak=message,
-            target=target,
-            skillId="amzn1.ask.1p.saysomething",
-            queue_delay=queue_delay,
-        )
+        if message.startswith("alexa.cannedtts.speak"):
+            await self.send_sequence(
+                "Alexa.CannedTts.Speak",
+                customerId=customer_id,
+                cannedTtsStringId=message,
+                skillId="amzn1.ask.1p.saysomething",
+                queue_delay=queue_delay,
+            )
+        else:
+            target = {
+                "customerId": customer_id,
+                "devices": self.process_targets(targets),
+            }
+            await self.send_sequence(
+                "Alexa.Speak",
+                customerId=customer_id,
+                textToSpeak=message,
+                target=target,
+                skillId="amzn1.ask.1p.saysomething",
+                queue_delay=queue_delay,
+            )
 
+    @_catch_all_exceptions
     async def send_announcement(
         self,
         message: Text,
@@ -622,6 +649,7 @@ class AlexaAPI:
             queue_delay=queue_delay,
         )
 
+    @_catch_all_exceptions
     async def send_mobilepush(
         self,
         message: Text,
@@ -669,36 +697,44 @@ class AlexaAPI:
             data=data,
         )
 
+    @_catch_all_exceptions
     async def previous(self) -> None:
         """Play previous."""
         await self.set_media({"type": "PreviousCommand"})
 
+    @_catch_all_exceptions
     async def next(self) -> None:
         """Play next."""
         await self.set_media({"type": "NextCommand"})
 
+    @_catch_all_exceptions
     async def pause(self) -> None:
         """Pause."""
         await self.set_media({"type": "PauseCommand"})
 
+    @_catch_all_exceptions
     async def play(self) -> None:
         """Play."""
         await self.set_media({"type": "PlayCommand"})
 
+    @_catch_all_exceptions
     async def forward(self) -> None:
         """Fastforward."""
         await self.set_media({"type": "ForwardCommand"})
 
+    @_catch_all_exceptions
     async def rewind(self) -> None:
         """Rewind."""
         await self.set_media({"type": "RewindCommand"})
 
+    @_catch_all_exceptions
     async def set_volume(self, volume: float, queue_delay: float = 1.5) -> None:
         """Set volume."""
         await self.send_sequence(
             "Alexa.DeviceControls.Volume", value=volume * 100, queue_delay=queue_delay
         )
 
+    @_catch_all_exceptions
     async def shuffle(self, setting: bool) -> None:
         """Shuffle.
 
@@ -706,6 +742,7 @@ class AlexaAPI:
         """
         await self.set_media({"type": "ShuffleCommand", "shuffle": setting})
 
+    @_catch_all_exceptions
     async def repeat(self, setting: bool) -> None:
         """Repeat.
 
@@ -756,6 +793,7 @@ class AlexaAPI:
         )
         return await response.json(content_type=None) if response else None
 
+    @_catch_all_exceptions
     async def set_bluetooth(self, mac: Text) -> None:
         """Pair with bluetooth device with mac address."""
         await self._post_request(
@@ -766,6 +804,7 @@ class AlexaAPI:
             data={"bluetoothDeviceAddress": mac},
         )
 
+    @_catch_all_exceptions
     async def disconnect_bluetooth(self) -> None:
         """Disconnect all bluetooth devices."""
         await self._post_request(
@@ -784,7 +823,9 @@ class AlexaAPI:
             "get", login, "/api/devices-v2/device", query=None
         )
         AlexaAPI.devices[login.email] = (
-            (await response.json(content_type=None))["devices"] if response else None
+            (await response.json(content_type=None))["devices"]
+            if response
+            else AlexaAPI.devices[login.email]
         )
         return AlexaAPI.devices[login.email]
 
@@ -834,6 +875,7 @@ class AlexaAPI:
         return await response.json(content_type=None) if response else None
 
     @staticmethod
+    @_catch_all_exceptions
     async def get_last_device_serial(
         login: AlexaLogin, items: int = 10
     ) -> Optional[Dict[Text, Any]]:
